#!/bin/sh

. "$_EXEC/indexmeta.sh"
. "$_EXEC/widgets.sh"

list_item() {
  local name path length width height tags comment n
  name="$(HTML "$1")"
  path="$(HTML "$ITEM/$1")"

  if [ -d "$_DATA/$ITEM/$1" ]; then
    printf '[a .list .dir href="%s?%s" %s]' \
      "$path" "$(HTML "$QUERY_STRING")" "$name"
  else
    read -r length width height tags comment n <<-EOF
	$(meta_info "$_DATA/$ITEM/$1")
	EOF
    printf '[div .list .file
              [a href="%s" [img src="%s?a=thumbnail"][label %s]]
              [span .time %i:%imin] [span .dim %ix%i] %s
              [checkbox "select" "%s" id="select_%s" form="multitag"][label for="select_%s" +]
            ]' \
      "$path" "$path" "$name" \
      "$((length / 60))" "$((length % 60))" \
      "$width" "$height" \
      "$(UNSTRING "${tags#tags=}" |tr , '\0' |xargs -r0 printf ' [span .tag %s]')" \
      "$path" "$path" "$path"
  fi
}

list_directories(){
  (cd "$_DATA/$ITEM";
    find ./ -type d \! -name .index -mindepth 1 -maxdepth 1 \
  ) | cut -d/ -f2- | sort
}

list_filemeta(){
  local meta base f fn file
  base="$1"
  meta="$1/.index/meta"
  meta_dir "$_DATA/$ITEM/$base"

  grep -iE "$(STRING "${SEARCH}")" "$_DATA/$ITEM/$meta" \
  | cut -f1,6- \
  | while f="$(line)"; do
    fn="$(UNSTRING "${f#*	}")"; fn="${fn%${CR}}";
    file="$(printf '%s\n' "$_DATA/$ITEM/$base/${fn}".*)"
    file="${file##*/}"
    [ -e "$_DATA/$ITEM/$base/${file}" ] \
    && printf '%s	%s\n' "${f%%	*}" "${base}/${file}"
  done
}

list_index(){
  local meta
  (cd "$_DATA/$ITEM";
    find ./ -path '*/.index/meta'
  ) | while meta="$(line)"; do
    list_filemeta "${meta%/.index/meta}"
  done
}

list_items() {
  local mode
  mode="$(COOKIE mode |grep -m1 -xE 'index|browse' || printf browse )"

  [ "$mode" = browse -a "$ITEM" ] && printf '..\n'

  if   [ "$mode" = browse -a "$ORDER" = Date ]; then
    list_directories
    list_filemeta . |cut -f2- \
    | xargs -d\n stat -c '%Y	%n' \
    | sort -rn |cut -d/ -f2-
  elif [ "$mode" = browse -a "$ORDER" = Name ]; then
    list_directories
    list_filemeta . \
    | sort -k 2 |cut -d/ -f2-
  elif [ "$mode" = index  -a "$ORDER" = Date ]; then
    list_index |cut -f2- \
    | xargs -d'\n' stat -c '%Y	%n' \
    | sort -rn | cut -d/ -f2-
  elif [ "$mode" = index  -a "$ORDER" = Name ]; then
    list_index | sort -k 2 | cut -d/ -f2-
  elif [ "$mode" = browse -a "$ORDER" = Length ]; then
    list_directories
    list_filemeta . \
    | sort -n |cut -d/ -f2-
  elif [ "$mode" = index  -a "$ORDER" = Length ]; then
    list_index \
    | sort -n |cut -d/ -f2-
  fi
}

list_paginate() {
  local page i c n
  page="$(GET p |grep -xE '[0-9]+' || printf 1)"

  printf '[div .itemlist '
  while i="$(line)"; do
    c=$((${c-0} + 1))
    if [ $c -lt $page ]; then
      true
    elif [ $c -lt $((LISTSIZE + page)) ]; then
      list_item "$i"
    fi
  done
  printf ']'

  printf '[div .pagination'
  for n in $(seq 1 $((c / LISTSIZE + 1)) ); do
    printf '[a .page href="%s" %s]' \
      "?p=$(( (n - 1) * LISTSIZE + 1))" "$n"
  done
  printf ']'
}

printf 'Content-Type: text/html;charset=utf-8\r\n\r\n'

"$_EXEC/cgilite/html-sh.sed" <<-EOF
[!DOCTYPE HTML]
[html [head [title Listing]
  [link rel=stylesheet href="/style.css" ]
] [body
  [div #navigation
    [a #t_bookmarks href="#bookmarks" &#x2605;]
    $(w_search)
    [a #t_avsearch href="#advsearch" Advanced]
    [a #t_prefs href="#prefs" &#x2699;]
  ]
  $(w_prefs)

  [form method=POST action="?a=multitag"
    $(list_items \
      | list_paginate
    )
    [div #editing
      [a href="#multitag" Add Tags] $(w_tagging)
      $(w_index)
    ]
  ]
] ]
EOF

